mkdir C:\users\mauro\opencpn\build\gshhs
copy C:\users\mauro\OpenCPN\data\gshhs\*.* C:\users\mauro\opencpn\build\gshhs

mkdir C:\users\mauro\opencpn\build\s57data
copy C:\users\mauro\OpenCPN\data\s57data\*.* C:\users\mauro\opencpn\build\s57data
  
mkdir C:\users\mauro\opencpn\build\tcdata
copy C:\users\mauro\OpenCPN\data\tcdata\*.* C:\users\mauro\opencpn\build\tcdata

mkdir C:\users\mauro\opencpn\build\uidata
copy C:\users\mauro\OpenCP\src\bitmaps\styles.xml C:\users\mauro\opencpn\build\uidata
copy C:\users\mauro\OpenCP\src\bitmaps\toolicons_journeyman_flat.png C:\users\mauro\opencpn\build\uidata
copy C:\users\mauro\OpenCP\src\bitmaps\toolicons_journeyman.png  C:\users\mauro\opencpn\build\uidata
copy C:\users\mauro\OpenCP\src\bitmaps\toolicons_traditional.png C:\users\mauro\opencpn\build\uidata