/* Autogenerated by png2wx.pl on Fri May 29 16:20:54 2015 */
#ifndef ICONS_H
#define ICONS_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_anchor_blu;
extern wxBitmap *_img_aton_gry;
extern wxBitmap *_img_club_pur;
extern wxBitmap *_img_fuelpump_red;
extern wxBitmap *_img_logimg_C;
extern wxBitmap *_img_logimg_E;
extern wxBitmap *_img_logimg_NE;
extern wxBitmap *_img_logimg_N;
extern wxBitmap *_img_logimg_NW;
extern wxBitmap *_img_logimg_SE;
extern wxBitmap *_img_logimg_S;
extern wxBitmap *_img_logimg_SW;
extern wxBitmap *_img_logimg_U;
extern wxBitmap *_img_logimg_W;
extern wxBitmap *_img_marina_grn;
extern wxBitmap *_img_pier_yel;
extern wxBitmap *_img_plugin_logo;
extern wxBitmap *_img_ramp_azu;

#endif /* ICONS_H */
